package javax.net.ssl;

import java.security.KeyManagementException;
import java.security.SecureRandom;
import java.security.NoSuchProviderException;
import sun.security.jca.GetInstance;
import java.security.NoSuchAlgorithmException;
import java.security.Provider;

public class SSLContext
{
    private final Provider provider;
    private final SSLContextSpi contextSpi;
    private final String protocol;
    private static SSLContext defaultContext;
    
    protected SSLContext(SSLContextSpi contextSpi, Provider provider, String protocol) {
        this.contextSpi = contextSpi;
        this.provider = provider;
        this.protocol = protocol;
    }
    
    public static synchronized SSLContext getDefault() throws NoSuchAlgorithmException {
        if (SSLContext.defaultContext == null) {
            SSLContext.defaultContext = getInstance("Default");
        }
        return SSLContext.defaultContext;
    }
    
    public static synchronized void setDefault(SSLContext defaultContext) {
        if (defaultContext == null) {
            throw new NullPointerException();
        }
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(new SSLPermission("setDefaultSSLContext"));
        }
        SSLContext.defaultContext = defaultContext;
        return;
    }
    
    public static SSLContext getInstance(String algorithm) throws NoSuchAlgorithmException {
        GetInstance.Instance instance = GetInstance.getInstance("SSLContext", SSLContextSpi.class, algorithm);
        return new SSLContext((SSLContextSpi)instance.impl, instance.provider, algorithm);
    }
    
    public static SSLContext getInstance(String algorithm, String provider) throws NoSuchAlgorithmException, NoSuchProviderException {
        GetInstance.Instance instance = GetInstance.getInstance("SSLContext", SSLContextSpi.class, algorithm, provider);
        return new SSLContext((SSLContextSpi)instance.impl, instance.provider, algorithm);
    }
    
    public static SSLContext getInstance(String algorithm, Provider provider) throws NoSuchAlgorithmException {
        GetInstance.Instance instance = GetInstance.getInstance("SSLContext", SSLContextSpi.class, algorithm, provider);
        return new SSLContext((SSLContextSpi)instance.impl, instance.provider, algorithm);
    }
    
    public final String getProtocol() {
        return this.protocol;
    }
    
    public final Provider getProvider() {
        return this.provider;
    }
    
    public final void init(KeyManager[] array, TrustManager[] array2, SecureRandom secureRandom) throws KeyManagementException {
        this.contextSpi.engineInit(array, array2, secureRandom);
    }
    
    public final SSLSocketFactory getSocketFactory() {
        return this.contextSpi.engineGetSocketFactory();
    }
    
    public final SSLServerSocketFactory getServerSocketFactory() {
        return this.contextSpi.engineGetServerSocketFactory();
    }
    
    public final SSLEngine createSSLEngine() {
        try {
            return this.contextSpi.engineCreateSSLEngine();
        }
        catch (AbstractMethodError cause) {
            UnsupportedOperationException ex = new UnsupportedOperationException("Provider: " + this.getProvider() + " doesn't support this operation");
            ex.initCause(cause);
            throw ex;
        }
    }
    
    public final SSLEngine createSSLEngine(String s, int n) {
        try {
            return this.contextSpi.engineCreateSSLEngine(s, n);
        }
        catch (AbstractMethodError cause) {
            UnsupportedOperationException ex = new UnsupportedOperationException("Provider: " + this.getProvider() + " does not support this operation");
            ex.initCause(cause);
            throw ex;
        }
    }
    
    public final SSLSessionContext getServerSessionContext() {
        return this.contextSpi.engineGetServerSessionContext();
    }
    
    public final SSLSessionContext getClientSessionContext() {
        return this.contextSpi.engineGetClientSessionContext();
    }
    
    public final SSLParameters getDefaultSSLParameters() {
        return this.contextSpi.engineGetDefaultSSLParameters();
    }
    
    public final SSLParameters getSupportedSSLParameters() {
        return this.contextSpi.engineGetSupportedSSLParameters();
    }
}
